import datetime
from matplotlib.dates import relativedelta


def check_date_not_tomorrow(date: datetime.datetime) -> bool:
    """
    Check if the given date is not tomorrow.

    Args:
        date (datetime.datetime): The date to check.

    Returns:
        bool: True if the date is not tomorrow, False otherwise.
    """
    today = datetime.datetime.today()
    if date > today:
        return False
    else:
        return True


def is_dates_diff_more_than_one_year(
    date1: datetime.datetime, date2: datetime.datetime
) -> bool:
    """
    Check if the difference between two dates is more than one year.

    Args:
        date1 (datetime.datetime): The first date.
        date2 (datetime.datetime): The second date.

    Returns:
        bool: True if the difference is more than one year, False otherwise.
    """
    if date2 - date1 > datetime.timedelta(days=365):
        return True
    else:
        return False


def calculate_dates_interval(
    date1: datetime.datetime, date2: datetime.datetime, time_type: str
) -> str:
    """
    Calculate the interval between two dates based on the given time type.

    Args:
        date1 (datetime.datetime): The first date.
        date2 (datetime.datetime): The second date.
        time_type (str): The type of time interval to calculate.

    Returns:
        str: The calculated time interval.
    """
    start_day = date1.strftime("%Y-%m-%d")

    if time_type == "yyyy-mm-dd":
        end_day = date2.strftime("%Y-%m-%d")
        time_interval = f"{start_day}T00:00Z/{end_day}T23:00Z"

    elif time_type == "yyyy-W":
        end_day = date2.strftime("%Y-%m-%d")
        time_interval = f"{start_day}T00:00Z/{end_day}T23:00Z"

    elif time_type == "yyyy-mm":
        end_day = (date2 + relativedelta(day=31)).strftime("%Y-%m-%d")
        start_day = (date1 - relativedelta(day=1)).strftime("%Y-%m-%d")
        time_interval = f"{start_day}T23:00Z/{end_day}T23:00Z"

    elif time_type == "yyyy":
        end_day = (date2 + relativedelta(yearday=365)).strftime("%Y-%m-%d")
        time_interval = f"{start_day}T23:00Z/{end_day}T23:00Z"

    return time_interval


def get_interval(date1: datetime.datetime, date2: datetime.datetime) -> str:
    """
    Get the interval between two dates in a specific format.

    Args:
        date1 (datetime.datetime): The first date.
        date2 (datetime.datetime): The second date.

    Returns:
        str: The interval between the two dates.
    """
    tmp_date1 = date1.strftime("%Y-%m-%dT%H:%MZ")
    tmp_date2 = date2.strftime("%Y-%m-%dT%H:%MZ")
    date = f"{tmp_date1}/{tmp_date2}"
    return date


def split_interval(interval: str) -> tuple:
    """
    Split the interval string into two datetime objects.

    Args:
        interval (str): The interval string.

    Returns:
        tuple: A tuple containing the two datetime objects.
    """
    dates = interval.split("/")
    date1 = datetime.datetime.strptime(dates[0], "%Y-%m-%dT%H:%MZ")
    date2 = datetime.datetime.strptime(dates[1], "%Y-%m-%dT%H:%MZ")
    return date1, date2


def get_format(time_type: str) -> str:
    """
    Get the date format based on the given time type.

    Args:
        time_type (str): The type of time interval.

    Returns:
        str: The date format.
    """
    time_format = ""
    if time_type == "yyyy-mm-dd":
        time_format = "%Y-%m-%d"
    elif time_type == "yyyy-W":
        time_format = "%Y-%W"
    elif time_type == "yyyy-mm":
        time_format = "%Y-%m"
    elif time_type == "yyyy-w":
        time_format = "%Y-%W"
    else:
        time_format = "%Y"

    return time_format


def date_diff(date1: datetime.datetime, date2: datetime.datetime) -> int:
    """
    Calculate the difference in days between two dates.

    Args:
        date1 (datetime.datetime): The first date.
        date2 (datetime.datetime): The second date.

    Returns:
        int: The difference in days.
    """
    diff = date2 - date1
    return diff.days


def get_week_boundrais(date: datetime.datetime, week_number: int) -> tuple:
    # calculate boundrais of the week_number-th week of the year
    year = date.year
    week = week_number
    initial_time_start = f"{year}-{week}-1"
    final_time_start = f"{year}-{week}-0"

    week_start = datetime.datetime.strptime(initial_time_start, "%Y-%W-%w")
    week_end = datetime.datetime.strptime(final_time_start, "%Y-%W-%w")

    return week_start, week_end


def count_weeks(start_date, end_date):
    """
    Counts the number of weeks between two dates.

    Args:
    start_date (datetime): The start date.
    end_date (datetime): The end date.

    Returns:
    int: The number of weeks between the two dates.
    """
    # Calculate the difference between the two dates
    delta = end_date - start_date

    # Divide the difference in days by 7 and round up to the nearest whole number
    number_of_weeks = (
        delta.days + 6
    ) // 7  # Adding 6 ensures that partial weeks are counted as full weeks

    return number_of_weeks



from rich.prompt import Prompt


def input_security_token() -> str:
    """
    Prompts the user to input a security token and saves it in a .env file.

    Returns:
        str: The security token inserted by the user.
    """

    security_token = str(
        Prompt.ask(
            f"Insert your security token. You can ask for one at [link=https://transparency.entsoe.eu/content/static_content/Static%20content/web%20api/Guide.html#_authentication_and_authorisation]this link[/link]",
        )
    ).lower()

    return security_token



from rich.prompt import Prompt
from entsopy.utils.const import *
from entsopy.components.table import create_table
from rich import print
import json
from entsopy.utils.utils import extract_code_from_key
from importlib import resources

def input_docstatus() -> str:
    """
    Prompts the user to select a document status from a list and returns the corresponding code.

    Returns:
        str: The code of the selected document status.
    """
    element = "DocStatus"
    data = json.load(resources.open_text("entsopy.data.types", "docstatus.json"))    

    table = create_table(
        [f"{element}", "Code", "Key"],
        title=f"Select the [b]{element}[/b] of the data you want to download from the list below",
        rows=data,
    )
    print(table)

    selected_docstatus = str(
        Prompt.ask(
            f"Insert the [b gold1]{element}[/b gold1] of the data you want to download",
            choices=[str(x["key"]) for x in data],
        )
    ).lower()

    docstatus = extract_code_from_key(data, selected_docstatus)
    return docstatus



import json
import pandas as pd
from dataclasses import dataclass
from lxml import etree
from entsopy.utils.const import *
from entsopy.utils.utils import *
from importlib import resources


@dataclass
class ResponseData:
    """
    Represents a response data object.

    Args:
        content (str): The content of the response.
        article_code (str): The article code.

    Attributes:
        root (etree.Element): The root element of the XML content.
        article_code (str): The article code.
        ns_name (str): The namespace name.
        nsmap (dict): The namespace map.
        data (list): The list of data rows.
        df (pd.DataFrame): The data in a pandas DataFrame.
        max_no_points (int): The maximum number of points.
        resolution (str): The resolution of the timeserie period.

    Methods:
        save_to_csv(file_name: str, path: str = ""): Saves the data to a CSV file.
        fill_missing_psr_types(all_psr_types: list = []): Fills missing PSR types in the data.
    """

    def __init__(self, content: str, article_code: str):
        content = content
        self.root = etree.XML(content)
        self.article_code = article_code
        self.ns_name = "ns"
        self.nsmap = get_namespace_from_root(self.root)

        data = []

        document_res = extract_elements_from_node(
            self.root,
            to_exclude=[
                "Point",
                "position",
                "quantity",
                "TimeSeries",
                "time_Period.timeInterval",
            ],
        )
        self.resolution = self.root.findtext(".//ns:resolution", namespaces=self.nsmap)
        self.date_start = datetime.strptime(
            document_res["doc.timeInterval.start"], "%Y-%m-%dT%H:%MZ"
        )
        self.date_end = datetime.strptime(
            document_res["doc.timeInterval.end"], "%Y-%m-%dT%H:%MZ"
        )
        timeseries = self.root.findall(
            f".//{self.ns_name}:TimeSeries", namespaces=self.nsmap
        )
        period = self.root.find(
            f".//{self.ns_name}:Period",
            namespaces=self.nsmap,
        )

        max_no_points = max_number_of_points(period, self.resolution, self.nsmap)

        for tms in timeseries:
            row = {}

            data_tms = extract_elements_from_node(
                tms, to_exclude=["Point", "position", "quantity"], prefix="tms"
            )

            for i in range(1, max_no_points + 1):
                data_point = {}

                data_timing = get_time_data(
                    date_start=self.date_start,
                    date_end=self.date_start,
                    resolution=self.resolution,
                    position=i,
                )

                data_point["position"] = i
                data_point["quantity"] = get_point_quantity(tms, i, self.nsmap)
                row = {
                    **document_res,
                    **data_tms,
                    **data_timing,
                    **data_point,
                }
                data.append(row)

        self.data = data
        self.df = pd.DataFrame(self.data)
        self.max_no_points = max_no_points

        if self.article_code == "14.1.A" or self.article_code == "16.1.B&C":
            self.fill_missing_psr_types()
        elif self.article_code == "14.1.D":
            self.fill_missing_psr_types(all_psr_types=["B16", "B18", "B19"])

    def save_to_csv(self, file_name: str, path: str = ""):
        timestamp = (datetime.now()).strftime("%Y%m%dT%H%M%S")
        self.df.to_csv(
            f"{file_name}-{self.article_code}-{timestamp}.csv",
            index=True,
            index_label="id",
        )

    def fill_missing_psr_types(
        self,
        all_psr_types: list = json.load(resources.open_text("entsopy.data.types", "psrtypes.json"))
    ):    
                
        psr_type = self.df["mkt.psrType"].unique()

        missing_psr_types = [psr for psr in all_psr_types if psr not in psr_type]

        for i in range(1, self.max_no_points + 1):
            for missing_psr_type in missing_psr_types:
                new_row = self.df[:1]
                new_row["quantity"] = "na"
                new_row["position"] = i
                new_row["mkt.psrType"] = missing_psr_type
                self.df = pd.concat([self.df, new_row])

                data_time = get_time_data(
                    self.date_start,
                    self.date_start,
                    position=i,
                    resolution=self.resolution,
                )

                for key, value in data_time.items():
                    new_row[key] = value

        if self.article_code == "14.1.A" or self.article_code == "16.1.B&C":
            self.df = self.df.sort_values(by=["mkt.psrType"])

        self.df = self.df.reset_index(drop=True)



import datetime
from rich.prompt import Prompt
from entsopy.utils.date import *


def input_date(
    time_type: str,
) -> str:
    """
    Prompts the user to input a start date and an end date with the specified time format.
    Validates that the end date is greater than the start date.
    Calculates the dates interval based on the input dates and time type.

    Args:
        time_type (str): The format of the dates to be inputted.
        time_range (str, optional): The range of time for the dates. Defaults to "".

    Returns:
        str: The calculated dates interval.
    """

    time_format = get_format(time_type)
    element = "start date"
    date_1 = str(
        Prompt.ask(
            f"Insert the [b gold1]{element}[/b gold1] with the format {time_type}",
        )
    ).lower()

    if time_format == "%Y-%W":
        time_format = "%Y-%W-%w"
        date_1 = date_1 + "-1"

    date_1 = datetime.datetime.strptime(date_1, f"{time_format}")
    date_2 = date_1 - datetime.timedelta(days=1)

    if time_format == "%Y-%W-%w":
        time_format = "%Y-%W"

    element = "end date"
    while date_2 < date_1:
        date_2 = str(
            Prompt.ask(
                f"Insert the [b gold1]{element}[/b gold1] with the format {time_type}",
            )
        ).lower()
        if time_format == "%Y-%W":
            time_format = "%Y-%W-%w"
            date_2 = date_2 + "-0"

        date_2 = datetime.datetime.strptime(date_2, f"{time_format}")

        if date_1 > date_2:
            print(
                f"[b red]The end date must be greater than the start date[/b red]. Please insert again the end date."
            )

    dates_interval = calculate_dates_interval(date_1, date_2, time_type)
    return dates_interval



from rich import print
from tkinter import filedialog


def input_download_directory() -> str:
    """
    Prompts the user to choise a directory in which downloaded files will be saved.

    Returns:
        str: The directory choosed by the user.
    """

    dir = None

    while not dir:
        print(
            "Please select a [b]directory[/b] in which downloaded files will be saved."
        )
        dir = filedialog.askdirectory()
        if not dir:
            print(
                f"The [red][b]selected directory is not valid[/red][/b] ({dir}). Please try again."
            )

    return dir



from datetime import timedelta
import requests
from requests.adapters import HTTPAdapter
import urllib3
from dataclasses import dataclass
from entsopy.classes.request import RequestData
from entsopy.classes.request import RequestData
from entsopy.utils.const import API_ENDPOINT
from entsopy.logger.logger import LOGGER
from entsopy.utils.date import split_interval, get_interval, date_diff


@dataclass
class HttpsClient:
    """
    A class representing an HTTPS client.

    Attributes:
        client (requests.Session): The HTTP client session.
        retry_policy (urllib3.Retry): The retry policy for HTTP requests.
        adapter (HTTPAdapter): The HTTP adapter for handling retries.
        security_token (str): The security token for authentication.
        api_endpoint (str): The API endpoint URL.

    Methods:
        __init__(self, security_token: str): Initializes the HttpsClient object.
        get_request(self, params: dict): Sends a GET request to the API endpoint.
        multiple_requests(self, request: RequestData) -> list: Sends multiple requests to the API endpoint.

    """

    client: requests.Session
    retry_policy: urllib3.Retry
    adapter: HTTPAdapter
    security_token: str
    api_endpoint = API_ENDPOINT

    def __init__(self, security_token: str):
        """
        Initializes the HttpsClient object.

        Args:
            security_token (str): The security token for authentication.

        """
        self.security_token = security_token
        self.client = requests.Session()
        self.retry_policy = urllib3.Retry(connect=15, backoff_factor=0.5, total=10)
        self.adapter = HTTPAdapter(max_retries=self.retry_policy)
        self.client.mount("http://", self.adapter)
        self.client.mount("https://", self.adapter)
        self.header = {
            "Content-Type": "application/xml",
            "SECURITY_TOKEN": self.security_token,
        }

    def get_request(self, params: dict):
        """
        Sends a GET request to the API endpoint.

        Args:
            params (dict): The parameters for the request.

        Returns:
            list: The response content.

        """
        params["securityToken"] = self.security_token
        response = self.client.get(url=self.api_endpoint, params=params)
        LOGGER.info(f"GET: {response.url}")
        return [response.content]

    def multiple_requests(self, request: RequestData) -> list:
        """
        Sends multiple requests to the API endpoint.

        Args:
            request (RequestData): The request data object.

        Returns:
            list: The response content.

        """
        request.params["securityToken"] = self.security_token
        res = []
        start_date, end_date = split_interval(interval=request.params["TimeInterval"])
        dates_diff = date_diff(start_date, end_date)
        delta = timedelta(days=request.article.range_limit)

        if dates_diff > delta.days:
            # count = 0
            while start_date < end_date:
                interval_starting_date = start_date
                interval_ending_date = start_date + delta
                if interval_ending_date > end_date:
                    interval_ending_date = end_date
                for i in request.areas:
                    request.set_custom_attribute_by_domain(value=i)

                    tmp_time_interval = get_interval(
                        interval_starting_date, interval_ending_date
                    )
                    request.set_custom_attribute("TimeInterval", tmp_time_interval)

                    # print(request.params)
                    response = self.client.get(
                        url=self.api_endpoint, params=request.params
                    )
                    LOGGER.info(f"MULTI-GET: {response.url}")
                    res.append(response.content)

                start_date += delta

            return res
        else:
            for i in request.areas:
                request.set_custom_attribute_by_domain(i)
                response = self.client.get(url=self.api_endpoint, params=request.params)
                LOGGER.info(f"GET: {response.url}")
                res.append(response.content)
        return res



from rich.panel import Panel
from entsopy.utils.const import *
from rich import print


def panel_success(
    message: str = f"File [file_name] sucessfully downloaded", file_name: str = ""
) -> None:
    """
    Display a success message with an optional file name.

    Args:
        message (str): The success message to display. The placeholder [file_name] can be used to indicate where the file name should be inserted.
        file_name (str, optional): The name of the file that was successfully downloaded. Defaults to "".

    Returns:
        None
    """

    if file_name != "":
        message = message.replace("[file_name]", file_name)

    print(
        Panel(
            f"[b][green]{message}[/green][/b]",
            highlight=True,
        )
    )
    return



from datetime import datetime, timedelta
import sys
from dateutil.relativedelta import relativedelta, SU
from entsopy.utils.date import count_weeks, get_week_boundrais


def sanitize_from_urn(tag: str) -> str:
    """
    Remove the URN prefix from a tag.

    Args:
        tag (str): The tag to sanitize.

    Returns:
        str: The sanitized tag.
    """
    return tag.split("}")[-1]


def is_in_list(tag: str, lists: list) -> bool:
    """
    Check if a tag is present in any of the lists.

    Args:
        tag (str): The tag to check.
        lists (list): The lists to search in.

    Returns:
        bool: True if the tag is found in any of the lists, False otherwise.
    """
    for lista in lists:
        if tag in lista:
            return True
    return False


def get_wellformed_tag(
    tag: str, parent_tag: str, parent_tag_to_exlude: list = [], ns_name: str = "ns"
) -> bool:
    """
    Get the well-formed tag based on the parent tag and exclusion list.

    Args:
        tag (str): The tag.
        parent_tag (str): The parent tag.
        parent_tag_to_exlude (list, optional): The list of parent tags to exclude. Defaults to [].
        ns_name (str, optional): The namespace name. Defaults to "ns".

    Returns:
        bool: The well-formed tag.
    """
    tag = sanitize_from_urn(tag)
    parent_tag = sanitize_from_urn(parent_tag)
    res = ""

    if parent_tag in parent_tag_to_exlude or parent_tag == "":
        res = f"{ns_name}:{tag}"
    else:
        res = f"{ns_name}:{parent_tag}/{ns_name}:{tag}"

    return res


def get_wellformed_key(
    suffix: str,
    tag: str,
    string_to_replace: str = "ns:",
    string_to_replace_with: str = "",
) -> str:
    """
    Get the well-formed key based on the suffix and tag.

    Args:
        suffix (str): The suffix.
        tag (str): The tag.
        string_to_replace (str, optional): The string to replace in the tag. Defaults to "ns:".
        string_to_replace_with (str, optional): The string to replace with. Defaults to "".

    Returns:
        str: The well-formed key.
    """
    tmp = tag.replace(string_to_replace, string_to_replace_with)
    tmp = tmp.replace("/", ".")
    return f"{suffix}.{tmp}"


def get_wellformed_tags(
    root, parent_to_exclude: list = [], lists_to_check: list[list] = [[]]
):
    """
    Get the well-formed tags from the root element.

    Args:
        root: The root element.
        parent_to_exclude (list, optional): The list of parent tags to exclude. Defaults to [].
        lists_to_check (list[list], optional): The lists to check for tag presence. Defaults to [[]].

    Returns:
        list: The well-formed tags.
    """
    res = []
    if root != None:
        for element in root.xpath(".//*"):
            tag = element.tag
            parent_tag = element.getparent().tag if element.getparent() != None else ""
            well_formed_tag = get_wellformed_tag(
                tag=tag, parent_tag=parent_tag, parent_tag_to_exlude=parent_to_exclude
            )
            is_tag_in_lists = is_in_list(
                well_formed_tag,
                lists_to_check,
            )
            if is_tag_in_lists == False:
                res.append(well_formed_tag)
                lists_to_check.append([well_formed_tag])
    return res


def get_xml_data(root, elements_list: [], nsmap: dict, suffix: str) -> dict:
    """
    Get the XML data from the root element.

    Args:
        root: The root element.
        elements_list (list): The list of elements to extract data from.
        nsmap (dict): The namespace map.
        suffix (str): The suffix for the keys.

    Returns:
        dict: The extracted XML data.
    """
    row = {}
    for element in elements_list:
        e = root.find(
            element,
            namespaces=nsmap,
        )
        if e != None:
            tmp = e.text.strip()
            if len(tmp) != 0:
                key = get_wellformed_key(suffix=f"{suffix}", tag=element)
                row[key] = tmp
        else:
            key = get_wellformed_key(suffix=f"{suffix}", tag=element)
            row[key] = "na"
    return row


def get_point_quantity(period, i, nsmap, ns_name: str = "ns"):
    """
    Get the quantity of a point in a period.

    Args:
        period: The period element.
        i: The position of the point.
        nsmap: The namespace map.
        ns_name (str, optional): The namespace name. Defaults to "ns".

    Returns:
        str: The quantity of the point.
    """
    res = ""
    is_point_existing = False
    point = period.xpath(
        f".//{ns_name}:Point/{ns_name}:position[text()='{i}']", namespaces=nsmap
    )

    if point != None and len(point) > 0:
        is_point_existing = True
        point = point[0]

    if is_point_existing == False:
        res = "na"
    else:
        res = (point.getparent().find(f".//{ns_name}:quantity", namespaces=nsmap)).text
    return res


def get_period_dates(period, nsmap, ns_name: str = "ns") -> tuple:
    """
    Get the start and end dates of a period.

    Args:
        period: The period element.
        nsmap: The namespace map.
        ns_name (str, optional): The namespace name. Defaults to "ns".

    Returns:
        tuple: The start and end dates of the period.
    """

    if period == None:
        print("Period is None")

    start = period.findtext(f".//{ns_name}:start", namespaces=nsmap)
    end = period.findtext(f".//{ns_name}:end", namespaces=nsmap)

    start_date = datetime.strptime(start, "%Y-%m-%dT%H:%MZ")
    end_date = datetime.strptime(end, "%Y-%m-%dT%H:%MZ")

    return start_date, end_date


def get_period_resolution(period, nsmap, ns_name: str = "ns"):
    """
    Get the resolution of a period.

    Args:
        period: The period element.
        nsmap: The namespace map.
        ns_name (str, optional): The namespace name. Defaults to "ns".

    Returns:
        str: The resolution of the period.
    """
    res = ""
    resolution = period.find(f".//{ns_name}:resolution", namespaces=nsmap)
    if resolution != None:
        res = resolution.text
    return res


def interval_divided_by_delta(
    start_date: datetime, end_date: datetime, resolution: str
):
    current_time = start_date
    number_of_deltas = 0

    if resolution == "P7D":
        return count_weeks(start_date, end_date)

    while current_time < end_date:
        current_time = current_time + get_resolution_relativedelta(
            resolution=resolution, multiplier=1, date=current_time
        )
        number_of_deltas += 1

    return number_of_deltas


def get_resolution_relativedelta(
    resolution: str, multiplier: int = 0, date: datetime = None
) -> relativedelta | int:
    """
    Get the relative delta based on the resolution.

    Args:
        resolution (str): The resolution.
        multiplier (int, optional): The multiplier for the relative delta. Defaults to 1.

    Returns:
        relativedelta: The relative delta.
    """
    rel_delta = timedelta()
    if resolution == "P1Y":
        rel_delta = relativedelta(years=(1 * multiplier))
    elif resolution == "PT1M":
        rel_delta = relativedelta(months=(1 * multiplier))
    elif resolution == "P7D":
        rel_delta = date.isocalendar()[1]
    elif resolution == "P1D":
        rel_delta = relativedelta(days=(1 * multiplier))
    elif resolution == "PT60M":
        rel_delta = relativedelta(minutes=(60 * (multiplier + 1)))
    elif resolution == "PT30M":
        rel_delta = relativedelta(minutes=(30 * multiplier))
    elif resolution == "PT15M":
        rel_delta = relativedelta(minutes=(15 * multiplier))

    return rel_delta


def max_number_of_points(period, resolution, nsmap, ns_name: str = "ns"):
    """
    Get the maximum number of points in a period.

    Args:
        period: The period element.
        nsmap: The namespace map.
        ns_name (str, optional): The namespace name. Defaults to "ns".

    Returns:
        int: The maximum number of points.
    """
    start, end = get_period_dates(period=period, nsmap=nsmap, ns_name=ns_name)
    number_of_points = interval_divided_by_delta(
        start_date=start, end_date=end, resolution=resolution
    )

    return number_of_points


def get_namespace_from_root(root, namespace_name: str = "ns"):
    """
    Get the namespace from the root element.

    Args:
        root: The root element.
        namespace_name (str, optional): The namespace name. Defaults to "ns".

    Returns:
        dict: The namespace.
    """
    nmsp = root.nsmap
    res = {}
    for key, value in nmsp.items():
        res[namespace_name] = value

    return res


def get_mtu(
    date: datetime,
    calculate_only_mtu: bool = False,
    prefix: str = "",
) -> str | dict:
    """
    Get the MTU (Measurement Time Unit) from a date.

    Args:
        date (datetime): The date.
        calculate_only_mtu (bool, optional): Whether to calculate only the MTU. Defaults to False.
        prefix (str, optional): The prefix for the MTU elements. Defaults to "".

    Returns:
        str | dict: The MTU as a string or a dictionary of MTU elements.
    """
    mtu_elements = {}
    # stringify mtu
    mtu = date
    mtu_elements[f"mtu.{prefix}mtu"] = mtu.strftime("%Y-%m-%dT%H:%MZ")

    mtu_elements[f"mtu.{prefix}year"] = f"{mtu.year}"
    mtu_elements[f"mtu.{prefix}month"] = f"{mtu.month:02d}"
    mtu_elements[f"mtu.{prefix}day"] = f"{mtu.day:02d}"
    mtu_elements[f"mtu.{prefix}week"] = mtu.isocalendar()[1]
    mtu_elements[f"mtu.{prefix}hour"] = f"{mtu.hour:02d}"
    mtu_elements[f"mtu.{prefix}minute"] = f"{mtu.minute:02d}"

    if calculate_only_mtu:
        return mtu_elements[f"{prefix}mtu"]
    else:
        return mtu_elements


def get_time_data(
    date_start: datetime,
    date_end: datetime,
    position: int,
    resolution: str,
) -> dict:
    """
    Get the time data from start and end dates.

    Args:
        date_start (datetime): The start date.
        date_end (datetime): The end date.

    Returns:
        dict: The time data.
    """

    if resolution == "P7D":
        date_start, date_end = get_week_boundrais(date_start, position)
        mtu_start = get_mtu(prefix="start", date=date_start)
        mtu_end = get_mtu(prefix="end", date=date_end)

    else:
        mtu_start = get_mtu(
            prefix="start",
            date=(
                date_start
                + get_resolution_relativedelta(
                    resolution=resolution, multiplier=(position - 1), date=date_start
                )
            ),
        )
        mtu_end = get_mtu(
            prefix="end",
            date=(
                date_end
                + get_resolution_relativedelta(
                    resolution=resolution, multiplier=position, date=date_end
                )
            ),
        )

    mtu = {**mtu_start, **mtu_end}
    return mtu


def extract_code_from_key(dict_list: [dict], key: str) -> str:
    """
    Extract the code from a list of dictionaries based on a key.

    Args:
        dict_list ([dict]): The list of dictionaries.
        key (str): The key to search for.

    Returns:
        str: The code extracted from the dictionary.
    """
    for d in dict_list:
        if d["key"] == key:
            return d["code"]
    return ""


def extract_elements_from_node(node, to_exclude=[], prefix="doc"):
    tags_to_exclude = to_exclude
    data = {}
    if node.getchildren() == []:
        key = prefix + "." + sanitize_from_urn(node.tag)
        data[key] = node.text
    else:
        for child in node.xpath(".//*"):
            tag = sanitize_from_urn(child.tag).strip()
            parent_tag = ""
            grandparent_tag = ""

            parent = child.getparent()
            grandparent = parent.getparent() if parent != None else None

            if parent:
                parent_tag = sanitize_from_urn(parent.tag).strip()

            if parent and grandparent:
                grandparent_tag = sanitize_from_urn(grandparent.tag).strip()

            if (
                tag not in tags_to_exclude
                and parent_tag not in tags_to_exclude
                and grandparent_tag not in tags_to_exclude
            ):
                if child.getchildren() == []:
                    key = prefix + "." + parent_tag + "." + tag
                    data[key] = child.text
            else:
                pass
    return data


def is_debug_active() -> bool:
    """Return if the debugger is currently active"""
    return hasattr(sys, "gettrace") and sys.gettrace() is not None



from rich.table import Table
from rich import print
from entsopy.utils.const import *
from entsopy.components.panels.fail import panel_fail


def logtable(
    filename: str,
) -> None:
    name = DIRS[f"{filename}"]
    rows = open(name, "r").readlines()

    if len(rows) <= 0:
        panel_fail(
            "No logs found...",
            f"Please make sure you have downloaded some data.",
        )
        return None

    else:
        table = Table(
            title="LOG FILE",
            expand=True,
            title_style="yellow bold",
            show_lines=True,
        )
        table.add_column("Date", justify="left")
        table.add_column("API Call", justify="left")

        for row in rows:
            date, api_call = row.split("] ")
            date = date.replace("[", "")
            api_call = api_call.replace("GET: ", "")
            table.add_row(date, api_call)

        print(table)



from dataclasses import dataclass
from entsopy.classes.article import Article


@dataclass
class RequestData:
    """
    Represents a request data object.

    Args:
        article (Article): The article object.
        areas (list): The list of areas.
        time_interval (str): The time interval.
        contract_market_agreement (str, optional): The contract market agreement. Defaults to None.
        direction (str, optional): The direction. Defaults to None.
        auction_type (str, optional): The auction type. Defaults to None.
    """

    def __init__(
        self,
        article: Article,
        areas: list,
        time_interval: str,
        contract_market_agreement: str = None,
        direction: str = None,
        auction_type: str = None,
    ):
        self.article = article
        self.areas = areas
        self.direction = direction
        self.params = {}

        for key, value in self.article.attributes.items():
            if value != (-1) and value != 0 and value is not None:
                self.params[key] = value

        self.params["TimeInterval"] = time_interval

        if contract_market_agreement is not None:
            self.params["Contract_MarketAgreement.Type"] = contract_market_agreement

        if auction_type is not None:
            self.params["Auction.Type"] = auction_type

    def __repr__(self) -> str:
        return f"RequestData(domain='{self.article.domain}', article='{self.article}', areas='{self.areas}', params='{self.params}')"

    def set_custom_attribute(self, param: str, value: str):
        """
        Sets a custom attribute.

        Args:
            param (str): The parameter name.
            value (str): The parameter value.
        """
        self.params[param] = value

    def set_custom_attribute_by_domain(self, value: str | dict):
        """
        Sets a custom attribute based on the article domain.

        Args:
            value (str | dict): The value to set based on the article domain.
        """
        if self.article.domain in "generation":
            self.params["In_Domain"] = value["code"]
        elif self.article.domain in "load":
            self.params["OutBiddingZone_Domain"] = value["code"]
        elif self.article.domain in "transmission":
            if (
                self.article.attributes["In_Domain"] == 1
                and self.article.attributes["Out_Domain"] == 1
            ):
                if self.article.area == "BZNS":
                    self.params["In_Domain"] = value["In_Domain"]
                    self.params["Out_Domain"] = value["Out_Domain"]
                else:
                    self.params["In_Domain"] = value["code"]
                    self.params["Out_Domain"] = value["code"]

            if self.direction == "export":
                self.switch_two_params("In_Domain", "Out_Domain")

        elif self.article.domain in "balancing":
            if self.article.attributes["ControlArea_Domain"] == 1:
                self.params["ControlArea_Domain"] = value["code"]
            elif (
                self.article.attributes["Acquiring_Domain"] == 1
                and self.article.attributes["Connecting_Domain"] == 1
            ):
                self.params["Acquiring_Domain"] = value["Acquiring_Domain"]
                self.params["Connecting_Domain"] = value["Connecting_Domain"]

    def switch_two_params(self, key1: str, key2: str):
        """
        Switches the values of two parameters.

        Args:
            key1 (str): The first parameter key.
            key2 (str): The second parameter key.
        """
        tmp = self.params[key1]
        self.params[key1] = self.params[key2]
        self.params[key2] = tmp



import logging
from entsopy.utils.const import *
import os
from importlib import resources

# check if the log file exists, if not create it and write an empty string
if not os.path.exists(DIRS["log"]):
    with open(DIRS["log"], "w") as file:
        file.write("")

logging.basicConfig(
    filename=DIRS["log"],
    format="[%(asctime)s] %(message)s",
    filemode="a+",
    force=True,
)
LOGGER = logging.getLogger()
LOGGER.setLevel(logging.INFO)



from dataclasses import dataclass


@dataclass
class Article:
    """
    Represents an article with its attributes.

    Attributes:
        attributes (dict): A dictionary containing the attributes of the article.
    """

    def __init__(self, article: dict):
        self.attributes = article.pop("attributes")

        for key, value in article.items():
            setattr(self, key, value)

    def __repr__(self):
        for key in self.__dict__:
            return f"Article({key}='{getattr(self, key)}')"



from entsopy.utils.const import *
from entsopy.components.table import create_table
from rich import print
import json
from rich.prompt import Prompt
from entsopy.utils.utils import extract_code_from_key
from importlib import resources


def input_auction_type(is_category: bool = False) -> str:
    element = "Auction Type"
    if is_category:
        f = resources.open_text("entsopy.data.types", "auctions_type.json")
        element = "Auction Category"
    else:
        f = resources.open_text("entsopy.data.types", "auctions_category.json")
        element = "Auction Type"

    data = json.load(f)

    table = create_table(
        [f"{element}", "Code", "Key"],
        title=f"Select the [b]{element}[/b] of the data you want to download from the list below",
        rows=data,
    )
    print(table)

    selected_auction = str(
        Prompt.ask(
            f"Insert the [b gold1]{element}[/b gold1] of the data you want to download",
            choices=[str(x["key"]) for x in data],
        )
    ).lower()

    auction = extract_code_from_key(data, selected_auction)
    return auction



"""
This module contains the file paths and API endpoint used in the entsopy package.
"""

DIRS = {
    "log": "std.log",
    "areas_control_area": "../data/areas/controlArea.json",
    "areas_bidding_zone": "../data/areas/biddingZone.json",
    "areas_border_bidding_zone": "../data/areas/borderBiddingZone.json",
    "areas_border_market_balancing_area": "../data/areas/borderMarketBalancingArea.json",
    "articles_balancing": "../data/articles/balancing.articles.json",
    "articles_generation": "../data/articles/generation.articles.json",
    "articles_load": "../data/articles/load.articles.json",
    "articles_ncm": "../data/articles/ncm.articles.json",
    "articles_outages": "../data/articles/outages.articles.json",
    "articles_transmission": "../data/articles/transmission.articles.json",
    "type_attributes": "../data/types/attributes.json",
    "type_auctions_type": "../data/types/auctions_type.json",
    "type_auctions_category": "../data/types/auctions_category.json",
    "type_directions": "../data/types/directions.json",
    "type_domains": "../data/types/domains.json",
    "type_market_agreement_contract": "../data/types/type_market_agreement.contract.json",
    "type_market_agreement_type": "../data/types/type_market_agreement.type.json",
    "type_psrtypes": "../data/types/psrtypes.json",
    "type_docstatus": "../data/types/docstatus.json",
    "type_market_product": "../data/types/market_product.json"
}

API_ENDPOINT = "https://web-api.tp.entsoe.eu/api?"


import json
from rich.prompt import Prompt
from entsopy.components.table import create_table
from entsopy.utils.const import *
from rich import print
from importlib import resources
import os


def input_domain() -> str:
    """
    Prompts the user to select a domain from a list and returns the selected domain.

    Returns:
        str: The selected domain.
    """

    element = "domain"
    data = []
    data = json.load(resources.open_text("entsopy.data.types", "domains.json"))

    table = create_table(
        ["Domain", "Code", "Key to press"],
        title=f"Select the [b gold1]{element}[/b gold1] of the data you want to download from the list below",
        rows=data,
    )
    print(table)

    domain = str(
        Prompt.ask(
            f"Insert the [b gold1]{element}[/b gold1] you want to download data from\n",
            choices=[str(x["key"]) for x in data],
        )
    ).lower()

    return domain



from entsopy.classes.article import Article
from entsopy.utils.const import *
from entsopy.components.table import create_table
from rich import print
import json
from rich.prompt import Prompt
from importlib import resources


def extract_article(articles, article_to_extract: str) -> Article:
    for art in articles:
        if art["key"] == article_to_extract:
            return Article(article=art)


def input_article(domain: str) -> Article:
    """
    Prompts the user to select an article based on the specified domain and returns the selected article.

    Args:
        domain (str): The domain for selecting the article.

    Returns:
        Article: The selected article.
    """

    element = "article"
    if domain == "1":
        f = resources.open_text("entsopy.data.articles", "load.articles.json")
    elif domain == "2":
        f = resources.open_text("entsopy.data.articles", "ncm.articles.json")
    elif domain == "3":
        f = resources.open_text("entsopy.data.articles", "transmission.articles.json")
    elif domain == "4":
        f = resources.open_text("entsopy.data.articles", "generation.articles.json")
    elif domain == "5":
        f = resources.open_text("entsopy.data.articles", "balancing.articles.json")
    elif domain == "6":
        f = resources.open_text("entsopy.data.articles", "outages.articles.json")
    data = json.load(f)

    table = create_table(
        ["Article", "Code", "Key"],
        title=f"Select the [b]{element}[/b] of the data you want to download from the list below",
        rows=data,
    )
    print(table)

    selected_article = str(
        Prompt.ask(
            f"Insert the code of the [b]{element}[/b] you want to download data from\n",
            choices=[str(x["key"]) for x in data],
        )
    ).lower()

    article = extract_article(articles=data, article_to_extract=selected_article)
    return article



from rich.prompt import Prompt
from entsopy.utils.const import *
from entsopy.components.table import create_table
from rich import print
from rich import print


def input_registeredsource() -> str:
    """
    Prompts the user to input the EIC code of the Registered Resource they want to download data from.

    Returns:
        str: The EIC code of the Registered Resource.
    """
    element = "EIC code"
    data = []

    ask_text = (
        f"Insert the [b gold1]{element}[/b gold1] of the Registered Resource you want to download data from\n. \nYou can find the list of approved EIC codes here: [link=https://www.entsoe.eu/data/energy-identification-codes-eic/eic-approved-codes/]entsoe.eu/data/energy-identification-codes-eic/eic-approved-codes[/link]",
    )

    registered_source = str(Prompt.ask(ask_text)).lower()

    return registered_source



from entsopy.utils.const import *
from entsopy.components.auction import input_auction_type
from entsopy.components.docstatus import input_docstatus
from entsopy.components.marketagreement import input_market_agreement
from entsopy.classes.article import Article
from entsopy.components.areas import input_areas
from entsopy.components.dates import input_date
from entsopy.components.direction import input_direction
from rich import print
import json
from entsopy.components.marketproduct import input_market_product
from entsopy.components.psrtype import input_psrtype
from entsopy.components.registeredresource import input_registeredsource
from importlib import resources

def ui_article(article: Article):
    """
    Function to handle the user interface for an article.

    Args:
        article (Article): The article object.

    Returns:
        Tuple: A tuple containing the values for areas, time_interval, contract_market_agreement,
        direction, auction_type, docstatus, psrtype, market_product, and registered_resource.
    """
    attributes = json.load(resources.open_text("entsopy.data.types", "attributes.json"))
    attributes.sort(key=lambda x: x["priority"], reverse=True)

    areas = None
    time_interval = None
    contract_market_agreement = None
    direction = None
    auction_type = None
    docstatus = None
    psrtype = None
    market_product = None
    registered_resource = None

    for attribute in attributes:
        attribute = attribute["name"]
        if attribute in article.attributes and article.attributes[attribute] == 1:
            if attribute == "TimeInterval":
                time_interval = input_date(article.time_type)
            elif (
                attribute == "OutBiddingZone_Domain"
                or attribute == "BiddingZone_Domain"
                or attribute == "ControlArea_Domain"
                or attribute == "In_Domain"
                or attribute == "Out_Domain"
                or attribute == "Acquiring_Domain"
                or attribute == "Connecting_Domain"
            ):
                areas = input_areas(article.area)

            elif attribute == "Contract_MarketAgreement.Type":
                contract_market_agreement = input_market_agreement()
                direction = input_direction()

            elif attribute == "Type_MarketAgreement.Type":
                contract_market_agreement = input_market_agreement(isType=True)

            elif attribute == "Auction.Type":
                auction_type = input_auction_type()

            elif attribute == "Auction.Category":
                auction_type = input_auction_type(isCategory=True)

            elif attribute == "DocStatus":
                docstatus = input_docstatus()

            elif attribute == "PsrType":
                psrtype = input_psrtype()

            elif attribute == "Standard_MarketProduct":
                market_product = input_market_product()

            elif attribute == "Original_MarketProduct":
                market_product = input_market_product(is_standard=False)

            elif attribute == "RegisteredResource":
                registered_resource = input_registeredsource()

    return (
        areas,
        time_interval,
        contract_market_agreement,
        direction,
        auction_type,
        docstatus,
        psrtype,
        market_product,
        registered_resource,
    )



from rich.panel import Panel
from rich import print


def welcome_panel() -> None:
    """
    Displays a welcome panel with information about ENTSOPY and a link to the official entso-e website.
    """
    print(
        Panel(
            "Welcome to [cornflower_blue]ENTSOPY[/cornflower_blue]: your assistant for downloading data from entso-e "
            "transparency platform.\nVisit the official entso-e website here: ["
            "link=https://transparency.entsoe.eu/]transparency.entsoe.eu[/link]",
            style="white",
            title="[b][cornflower_blue]ENTSOPY[/cornflower_blue][/b]",
            title_align="center",
        )
    )



from rich.panel import Panel
from rich import print


def panel_fail(
    message: str = "Something went wrong, file not downloaded",
    error_description: str = "No error description provided",
) -> None:
    """
    Display a failure panel with the provided message and error description.

    Args:
        message (str, optional): The message to display in the failure panel. Defaults to "Something went wrong, file not downloaded".
        error_description (str, optional): The description of the error. Defaults to "No error description provided".

    Returns:
        None
    """

    print(
        Panel(
            f"[b][red]{message}...[/red][/b]\nError: {error_description}.\n",
            title="[b][red]FATAL ERROR![/red][/b]",
            highlight=True,
            title_align="center",
        )
    )
    return



from typing import List, Any
import typer
from entsopy.utils.const import *
from entsopy.components.table import create_table
from rich import print
import json
from rich.prompt import Prompt
from importlib import resources


def input_areas(area: str) -> list[Any]:
    """
    Prompts the user to input areas based on the given area type.

    Args:
        area (str): The type of area to select.

    Returns:
        list[Any]: A list of selected areas.

    """
    end = False
    selected_areas = []
    element = ""
    areas = []

    if area == "CTA":
        f = resources.open_text("entsopy.data.areas", "controlArea.json")
        element = "control area"
    elif area == "BZN":
        f = resources.open_text("entsopy.data.areas", "biddingZone.json")
        element = "bidding zone"
    elif area == "BZNS":
        f = resources.open_text("entsopy.data.areas", "borderBiddingZone.json")
        element = "border bidding zone"
    elif area == "MBAS":
        f = resources.open_text("entsopy.data.areas", "borderMarketBalancingArea.json")
        element = "border market balance area"

    areas = json.load(f)

    table = create_table(
        [f"{element.capitalize()}", "Code", "Key"],
        title=f"Select the [b]{element}[/b] of the data you want to download from the list below",
        rows=areas,
    )
    print(table)

    while not end:
        key = str(
            Prompt.ask(
                f"Insert the [b gold1]{element}[/b gold1] of the data you want to download",
                choices=[str(x["key"]) for x in areas],
            )
        ).lower()

        for area in areas:
            if key == area["key"]:
                tmp_area = area

        if tmp_area:
            selected_areas.append(tmp_area)
            areas = [area for area in areas if area["key"] != tmp_area["key"]]

            if len(areas) > 0:
                end = not (typer.confirm(f"Do you want to add another {element}?"))

                if not end:
                    table = create_table(
                        ["Control Area", "Code"],
                        title=f"Select one of the remaining {element} of the data you want to download from the list below",
                        rows=areas,
                    )
            else:
                print(f"No more {element}s available !")
                end = True

        else:
            print(
                f"[b][red]The {element} you inserted is not available![/red][b] Insert another one."
            )

    return selected_areas



import os
from typing import Annotated
import typer
from entsopy.components.panels.fail import panel_fail
from entsopy.components.panels.success import panel_success
from entsopy.components.welcome import welcome_panel
from entsopy.components.domain import input_domain
from entsopy.components.home import home
from entsopy.components.welcome import welcome_panel
import dotenv
from entsopy.components.securitytoken import input_security_token
from entsopy.components.logging.logtable import logtable
from entsopy.utils.const import *
from entsopy.components.downloaddirectory import input_download_directory
from entsopy.utils.utils import is_debug_active
from entsopy.logger.logger import LOGGER
from entsopy.classes.httpsclient import HttpsClient
import traceback
import configparser

""" Main module of the app. """


app = typer.Typer(
    help="""Welcome to ENTSOPY your assistant for dowloading data from entso-e transparency platform.""",
)


@app.command(help="Start Entsopy App")
def start():
    try:
        
        package_directory = os.path.dirname(__file__)
        config_file_path = f"{package_directory}/conf.ini"

        config = configparser.ConfigParser()
        config.read(config_file_path)
        
        #check if conf.ini file exists
        if not os.path.exists(config_file_path):
            with open(config_file_path, "w") as file:
                file.write("# conf.ini file created")
        
        config = configparser.ConfigParser()
        config.read(config_file_path)
        
        # check if conf.ini file has the required sections   
        if config.has_section('configuration') == False:
            # create the configuration section
            config.add_section('configuration')
            with open(config_file_path, 'w') as configfile:
                config.write(configfile)


        # check if conf.ini file has the required "security_token" and "download_dir" keys
        token = None
        if config.has_option('configuration', 'security_token'):
            token = config.get('configuration', 'security_token')

        download_dir = None
        if config.has_option('configuration', 'download_dir'):
            download_dir = config.get('configuration', 'download_dir')


        if token is None:
            token = input_security_token()
            # save token to conf.ini file
            config.set('configuration', 'security_token', token)
            

        if download_dir is None:
            download_dir = input_download_directory()
            # save download_dir to conf.ini file
            config.set('configuration', 'download_dir', download_dir)
            
        with open(config_file_path, 'w') as configfile:
            config.write(configfile)
            
        client = HttpsClient(token)

        welcome_panel()

        domain = input_domain()
        
        res = home(client=client, domain=domain, download_dir=download_dir)
        
        print (f"res: {res}")

        if res:
            panel_success(file_name=res)
        else:
            panel_fail()

    except Exception as e:
        panel_fail("Error!", f"{e}\n{traceback.format_exc()}.")
        LOGGER.info(f"ERROR: {e}.\nTraceback: {traceback.format_exc()}")


@app.command(
    "reset",
    help="Reset the security token, target download directory or clear the log file. Args aviable: security-token, download-dir, log, all.",
)
def reset(
    command: Annotated[
        str,
        typer.Argument(help="The name of reset action to perform."),
    ] = "",
):
    if command == "security-token" or command == "all":
        input_security_token()
        panel_success("Security token successfully replaced and log file cleared.")
    elif command == "download-dir" or command == "all":
        os.environ["DOWNLOAD_DIR"] = ""
        panel_success("Download directory successfully reset.")
    elif command == "log" or command == "all":
        open(DIRS["log"], "w").close()
        panel_success("Log file succesfully cleared")
    else:
        panel_fail("Command not recognized. Type reset --help for more info.")


@app.command(
    "log",
    help="Manage logs. Args aviable: show, clear.",
    short_help="Show logs of the app",
)
def log(
    command: Annotated[
        str,
        typer.Argument(help="The name of thea action to perform related to logs."),
    ] = "show",
):
    if command == "clear":
        open(DIRS["log"], "w").close()
        panel_success("Log file cleared")
    else:
        logtable("log")


if __name__ == "__main__":
    if is_debug_active():
        typer.run(start)
    else:
        app()



from rich.prompt import Prompt
from entsopy.utils.const import *
from entsopy.components.table import create_table
from rich import print
import json
from rich import print
from entsopy.utils.utils import extract_code_from_key
from importlib import resources

def input_market_product(is_standard: bool = True) -> str:
    """
    Prompts the user to select a market product and returns the corresponding code.

    Args:
        is_standard (bool, optional): Specifies whether the market product is standard or original. Defaults to True.

    Returns:
        str: The code of the selected market product.
    """

    element = ""
    data = []
    if is_standard:
        element = "Standard Market Product"
    else:
        element = "Original Market Product"
        
    data = json.load(resources.open_text("entsopy.data.types", "market_product.json"))

    table = create_table(
        [f"{element.capitalize()}", "Code" "Key"],
        title=f"Select the [b]{element}[/b] of the data you want to download from the list below",
        rows=data,
    )
    print(table)

    selected_market_product = str(
        Prompt.ask(
            f"Insert the {element} you want to download data from\n",
            choices=[str(x["key"]) for x in data],
        )
    ).lower()

    market_product = extract_code_from_key(data, selected_market_product)

    return market_product



from rich.table import Table


def create_table(
    headers: list,
    title: str,
    rows: list,
) -> Table:
    """
    Create a table with the given headers, title, and rows.

    Args:
        headers (list): The list of column headers.
        title (str): The title of the table.
        rows (list): The list of rows, where each row is a dictionary.

    Returns:
        Table: The created table.

    """
    table = Table(
        *headers, expand=True, title=title, title_style="yellow bold", show_lines=True
    )

    for row in rows:
        table.add_row(
            row["name"],
            row["code"] if "code" in row else "",
            row["key"]
            if "is_available" in row and row["is_available"] == True
            else (row["key"] if "is_available" not in row else ":x:"),
        )

    return table



import pandas as pd
from datetime import datetime


def concat_and_save_dfs(
    dfs: list,
    file_name: str,
    suffix: str = "",
    timestamp: str = (datetime.now()).strftime("%Y%m%dT%H%M%S"),
    download_dir: str = "",
) -> str:
    """
    Concatenates a list of DataFrames and saves the result as a CSV file.

    Args:
        dfs (list): A list of DataFrames to be concatenated.
        file_name (str): The base name of the output file.
        suffix (str, optional): A suffix to be added to the file name (default: "").
        timestamp (str, optional): A timestamp to be added to the file name (default: current timestamp).

    Returns:
        str: The name of the saved CSV file.

    """

    res = pd.concat(dfs, join="outer").fillna("na")
    timestamp = (datetime.now()).strftime("%Y%m%dT%H%M%S")
    file_saving_name = f"{file_name}-{suffix}-{timestamp}.csv"
    print(f"{download_dir}/{file_saving_name}")
    res.to_csv(f"{download_dir}/{file_saving_name}", index=True, index_label="id")
    return file_saving_name



from entsopy.classes.request import RequestData
from entsopy.components.article import input_article
from entsopy.components.ui import ui_article
from entsopy.utils.csv import concat_and_save_dfs
from entsopy.classes.response import ResponseData
from entsopy.classes.httpsclient import HttpsClient
from rich import print


def home(client: HttpsClient, domain: str, download_dir: str) -> str:
    """
    Executes the main flow of the program.

    Args:
        client (HttpsClient): The HTTPS client used to make requests.
        domain (str): The domain to be used for the request.

    Returns:
        str: The file name of the saved response.
    """

    article = input_article(domain=domain)
    (
        areas,
        time_interval,
        contract_market_agreement,
        direction,
        auction_type,
        docstatus,
        psrtype,
        market_product,
        registered_resource,
    ) = ui_article(article=article)

    print("[i]Generate the request...[/i]")

    request = RequestData(
        article=article,
        time_interval=time_interval,
        contract_market_agreement=contract_market_agreement,
        direction=direction,
        auction_type=auction_type,
        areas=areas,
    )
    print("[i]Sending the request...[/i]")

    data = client.multiple_requests(request=request)

    print("[i]Processing the response...[/i]")

    res = [
        (ResponseData(content, article_code=request.article.code)).df
        for content in data
    ]

    print("[i]Saving the response file...[/i]")
    file_name = concat_and_save_dfs(
        dfs=res,
        file_name=article.domain,
        suffix=article.code,
        download_dir=download_dir,
    )

    return file_name
